import React, {
  createContext,
  ReactElement,
  ReactNode,
  useEffect,
  useReducer,
  useRef,
} from "react";
import Reducer from "./CartReducer";
import { ContextType, CartType } from "./CartTypes";

/**
 * React Context-based Global Store with a reducer
 * and persistent saves to sessionStorage/localStorage
 **/
export function CartStore({
  children,
}: {
  children: ReactNode;
}): ReactElement {
  const [cartState, dispatch] = useReducer(Reducer, initializeState());
  const initialRenderCartState = useRef(true);
  const initialRenderPersistenceType = useRef(true);

  useEffect(() => {
    /*
      populate either sessionStorage or localStorage
      data from cartState based on persistenceType
    */
    if (initialRenderCartState.current) {
      initialRenderCartState.current = false;
      return;
    }
    const getPersistenceType = cartState.persistenceType;
    if (getPersistenceType === "sessionStorage") {
      sessionStorage.setItem("cartState", JSON.stringify(cartState));
    } else if (getPersistenceType === "localStorage") {
      localStorage.setItem("cartState", JSON.stringify(cartState));
    }
  }, [cartState]);

  useEffect(() => {
    /*
      purge sessionStorage or localStorage when either is selected
    */
    if (initialRenderPersistenceType.current) {
      initialRenderPersistenceType.current = false;
      return;
    }
    const getPersistenceType = cartState.persistenceType;
    if (getPersistenceType === "sessionStorage") {
      localStorage.removeItem("cartState");
    } else if (getPersistenceType === "localStorage") {
      sessionStorage.removeItem("cartState");
    }
  }, [cartState.persistenceType]);

  return (
    <cartContext.Provider value={{ cartState, dispatch }}>
      {children}
    </cartContext.Provider>
  );
};

export const cartContext = createContext({} as ContextType);

export const initialState: CartType = {
  cart: [],
    showCart: false,
    persistenceType: "sessionStorage",
  totalPrice: 0
};

function initializeState() {
  /*
    the order in which the data is compared is very important;
    first try to populate the state from Storage if not return initialState
  */

  if (typeof Storage !== "undefined") {
  } else {
    throw new Error("You need to enable Storage to run this app.");
  }

  const fromLocalStorage = JSON.parse(
    localStorage.getItem("cartState") as string
  );
  const fromSessionStorage = JSON.parse(
    sessionStorage.getItem("cartState") as string
  );
  return fromSessionStorage || fromLocalStorage || initialState;
}
