import { useState, useEffect } from 'react';
import { CollectionIcon, EmojiSadIcon } from '@heroicons/react/outline';

import { Product } from '../cart/CartTypes';
import { getItems, updateStock } from '../apis/back';

const Stock = () => {
  let [search, setSearch] = useState("");
  let [productsList, setProductsList] = useState<Product[]>([]);
  let [rawProductList, setRawProductList] = useState<Product[]>([]);
  let [oldQuantity, setOldQuantity] = useState((0));
  
  useEffect(() => {
    // Filter products by name with the search term
    setProductsList(rawProductList.filter(product => {
      return product.name.toLowerCase().includes(search.toLowerCase());
    }));
  }, [search, rawProductList]);

  useEffect(() => {
    getItems().then((items) => {
      setRawProductList(items);
      setProductsList(items);
    }).catch((_) => {
      setRawProductList([]);
      setProductsList([]);
    });
  }, []);

  const updateProductQuantity = (productId, quantity) => {
    let productsTemp = productsList;
    let product = productsTemp[productId];
    product.stock = quantity;
    setProductsList([...productsTemp]);
  };

  const toggleEditMode = (productId) => {
    let productsTemp = productsList;
    let product = productsTemp[productId];
    product.edit = product.edit === true ? false : true;
    
    // Set all product.edit in productsTemp to false
    productsTemp.forEach((p, idx) => {
      if (p.edit && p.edit === true && idx !== productId) {
        p.edit = false;
      }
    });

    productsTemp[productId] = product;
    setProductsList([...productsTemp]);

    if (product.edit === false) {
      // console.log("oldQuantity: ",oldQuantity);
      updateStock(product.id, product.stock - oldQuantity).then(() => {
        console.log("Save product: " + product.name + " with quantity: " + product.stock);
      }).catch((_) => {
        console.log("Error update stock BACK");
      });
    }else{
      setOldQuantity(product.stock);
    }
  };

  const getPrice = (product: Product):string => {
    let price = `${product.price.toFixed(2).replace(".", ",")} €`;
    if (product.subscriberPrice !== undefined) {
      price += ` / ${product.subscriberPrice.toFixed(2).replace(".", ",")} €`;
    } else {
      // Display the same price if there is not subscription price
      price += ` / ${product.price.toFixed(2).replace(".", ",")} €`;
    }
    return price;
  };

  return (
    <div className="max-w-7xl mx-auto px-4 sm:px-6 md:px-8">
      {rawProductList.length > 0 &&
        <div className="max-w-7xl mx-auto px-4 sm:px-6 md:px-8 pb-5">
          <label htmlFor="search" className="block text-sm font-medium text-gray-700">Recherche</label>
          <div className="mt-1 relative flex items-center">
            <input
              type="text"
              name="search"
              id="search"
              value={search}
              onChange={(e) => { setSearch(e.target.value) }}
              className="shadow-sm focus:ring-indigo-500 focus:border-indigo-500 block w-full pr-12 sm:text-sm border-gray-300 rounded-md"
            />
          </div>
        </div>
      }
      <div className="max-w-7xl mx-auto">
        {productsList.length === 0
          ? <div className="text-center w-full mt-10">
            <EmojiSadIcon className="mx-auto h-12 w-12 text-gray-400" />
            <h3 className="mt-2 text-sm font-medium text-gray-900">Aucun produit trouvé !</h3>
            {rawProductList.length > 0 &&
              <>
                <p className="mt-1 text-sm text-gray-500">Essayez de réduire vos critères de recherche.</p>
                <div className="mt-6">
                  <button
                    type="button"
                    onClick={() => setSearch("")}
                    className="inline-flex items-center px-4 py-2 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500"
                  >
                    <CollectionIcon className="-ml-1 mr-2 h-5 w-5" aria-hidden="true" />
                    Afficher tout
                  </button>
                </div>
              </>
            }
          </div>
          : <div className="shadow overflow-y-auto border-b border-gray-200 sm:rounded-lg">
            <table className="min-w-full divide-y divide-gray-200">
              <thead className="bg-gray-50">
                <tr>
                  <th
                    scope="col"
                    className="px-4 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Produit
                  </th>
                  <th
                    scope="col"
                    className="px-4 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Catégorie
                  </th>
                  <th
                    scope="col"
                    className="px-4 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Prix / Adhérent
                  </th>
                  <th
                    scope="col"
                    className="px-4 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Quantité
                  </th>
                  <th scope="col" className="pr-4 py-3">
                    <span className="sr-only">Éditer</span>
                  </th>
                </tr>
              </thead>
              <tbody>
                {productsList.map((product, productIdx) => (
                  <tr key={product.name} className={productIdx % 2 === 0 ? 'bg-white' : 'bg-gray-50'}>
                    <td className="px-4 py-4 whitespace-nowrap text-sm font-medium text-gray-900">{product.name}</td>
                    <td className="px-4 py-4 whitespace-nowrap text-sm text-gray-500">{product.category}</td>
                    <td className="px-4 py-4 whitespace-nowrap text-sm text-gray-500">{getPrice(product)}</td>
                    <td className="px-4 py-4 whitespace-nowrap text-sm text-gray-500">
                      {product.edit ?
                        <input
                          className="shadow-sm focus:ring-indigo-500 focus:border-indigo-500 sm:max-w-xs sm:text-sm border-gray-300 rounded-md"
                          onChange={(e) => { updateProductQuantity(productIdx, e.target.value) }}
                          type="number"
                          value={product.stock}
                        />
                        : <span>{product.stock}</span>
                      }
                    </td>
                    <td className="pr-4 py-4 whitespace-nowrap text-right text-sm font-medium">
                      <button onClick={() => { toggleEditMode(productIdx) }} type="button" className="inline-flex items-center px-3 py-2 border border-transparent text-sm leading-4 font-medium rounded-md text-indigo-700 bg-indigo-100 hover:bg-indigo-200 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
                        {product.edit ? "Sauvegarder" : "Éditer"}
                      </button>
                    </td>
                  </tr>
                ))}
              </tbody>
            </table>
          </div>}
      </div>
    </div>
  );
};

export default Stock;
