import { useState, useEffect } from 'react';
import { CollectionIcon, EmojiSadIcon } from '@heroicons/react/outline';

import { User, getUsers } from '../apis/student';

const Users = () => {
  let [search, setSearch] = useState("");
  let [rawUsers, setRawUsers] = useState<User[]>([]);
  let [usersList, setUsersList] = useState<User[]>([]);

  useEffect(() => {
    // Filter users by nom or prenom with search term
    let filteredUsers = rawUsers.filter(user => {
      return user.nom.toLowerCase().includes(search.toLowerCase())
        || user.prenom.toLowerCase().includes(search.toLowerCase())
        || user.email.toLowerCase().includes(search.toLowerCase())
        || user.studentNumber.toString().toLowerCase().includes(search.toLowerCase());
    });
    setUsersList(filteredUsers);
  }, [search, rawUsers]);

  useEffect(() => {
    // Fetch users from API
    getUsers().then((users) => {
      //console.log(users);
      setRawUsers(users);
      setUsersList(users);
    }).catch((_) => {
      setRawUsers([]);
      setUsersList([]);
    });
  }, []);

  return (
    <div className="max-w-7xl overflow-y-hidden mx-auto px-4 sm:px-6 md:px-8">
      {rawUsers.length > 0 &&
        <div className="max-w-7xl mx-auto px-4 sm:px-6 md:px-8 pb-5">
          <label htmlFor="search" className="block text-sm font-medium text-gray-700">Recherche</label>
          <div className="mt-1 relative flex items-center">
            <input
              type="text"
              name="search"
              id="search"
              value={search}
              onChange={(e) => { setSearch(e.target.value) }}
              className="shadow-sm focus:ring-indigo-500 focus:border-indigo-500 block w-full pr-12 sm:text-sm border-gray-300 rounded-md"
            />
          </div>
        </div>
      }
      <div className="max-w-7xl mx-auto">
        {usersList.length === 0
          ? <div className="text-center w-full mt-10">
            <EmojiSadIcon className="mx-auto h-12 w-12 text-gray-400" />
            <h3 className="mt-2 text-sm font-medium text-gray-900">Aucun utilisateur trouvé !</h3>
            {rawUsers.length > 0 &&
              <>
                <p className="mt-1 text-sm text-gray-500">Essayez de réduire vos critères de recherche.</p>
                <div className="mt-6">
                  <button
                    type="button"
                    onClick={() => setSearch("")}
                    className="inline-flex items-center px-4 py-2 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500"
                  >
                    <CollectionIcon className="-ml-1 mr-2 h-5 w-5" aria-hidden="true" />
                    Afficher tout
                  </button>
                </div>
              </>
            }
          </div>
          : <div className="shadow overflow-y-hidden border-b border-gray-200 sm:rounded-lg">
            <table className="min-w-full overflow-y-auto divide-y divide-gray-200">
              <thead className="bg-gray-50">
                <tr>
                  <th
                    scope="col"
                    className="px-6 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Nom
                  </th>
                  <th
                    scope="col"
                    className="px-6 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Email
                  </th>
                  <th
                    scope="col"
                    className="px-6 py-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Numéro étudiant
                  </th>
                  <th
                    scope="col"
                    className="py-3 pr-3 text-left text-xs font-medium text-gray-500 uppercase tracking-wider"
                  >
                    Adhérent?
                  </th>
                </tr>
              </thead>
              <tbody>
                {usersList.map((user, userIdx) => (
                  <tr key={user.studentNumber} className={userIdx % 2 === 0 ? 'bg-white' : 'bg-gray-50'}>
                    <td className="px-6 py-4 whitespace-nowrap text-sm font-medium text-gray-900">
                      <div className="flex items-center">
                        <div className="flex-shrink-0 h-10 w-10">
                          <img className="h-10 w-10 rounded-full" src={user.picture} alt="" />
                        </div>
                        <div className="ml-4">
                          <div className="text-sm font-medium text-gray-900">{user.nom.toUpperCase()}</div>
                          <div className="text-sm text-gray-500">{user.prenom}</div>
                        </div>
                      </div>
                    </td>
                    <td className="px-6 py-4 whitespace-nowrap text-sm text-gray-500">{user.email}</td>
                    <td className="px-6 py-4 whitespace-nowrap text-sm text-gray-500">{user.studentNumber}</td>
                    <td className="py-4 whitespace-nowrap text-sm text-gray-500">
                      {user.adherant
                        ? <span className="inline-flex items-center px-3 py-0.5 rounded-full text-sm font-medium bg-green-100 text-green-800">
                          Oui
                        </span>
                        : <span className="inline-flex items-center px-3 py-0.5 rounded-full text-sm font-medium bg-red-100 text-red-800">
                          Non
                        </span>
                      }</td>
                  </tr>
                ))}
              </tbody>
            </table>
          </div>}
      </div>
    </div>
  );
};

export default Users;
